///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Dummy Author
// Dummy company
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DSDummyTypeCAppl.cpp
//

#include <stdio.h>
#include <momusys.h>
#include <vopio.h>

#include "ClientApplications.h"     /* Naming Convention */
#include "DescriptionSchemes.h"     /* Naming Convention */
#include "ExtractionUtilities.h"    /* Naming Convention */
#include "CodingSchemes.h"          /* Naming Convention */
#include "Decoders.h"               /* Naming Convention */
#include "SearchUtilities.h"        /* Naming Convention */
#include "Media.h"
#include "Utilities.h"

using namespace XM;

const UUID DSDummyTypeClient::myID = UUID("");
const char *DSDummyTypeClient::myName = "DSDummyTypeClient";

//=============================================================================
DSDummyTypeClient::DSDummyTypeClient():
RamDB(),
NoOfElements(0),
DBDescription(0)
{
}

//----------------------------------------------------------------------------
DSDummyTypeClient::~DSDummyTypeClient()
{
  // Free memories
  RamDB.FreeDatabase();
  if (DBDescription) delete [] DBDescription;
}

//----------------------------------------------------------------------------
unsigned long DSDummyTypeClient::Open(char *ListFile, char *DDLFile)
{
  // Check that the file names are ok
  assert(ListFile  != NULL);
  assert(DDLFile != NULL);

  // -------------------- Initialisation of data ----------------------------

  // This sets up the dummy type descriptor interface
  GenericDSInterfaceABC* theDescriptionInterface = NULL;

  // This sets up the coding scheme for this D/DType and the interface
  GenericDSCS*                     theCS                  = NULL;
  GenericDSCSInterfaceABC*         theCSInterface         = NULL;

  // Ok, make new instatiations of these datatypes
  try{
    // Allocate memory for the coding scheme
    theCS=new GenericDSCS();

    // Obtain the interface
    theCSInterface=theCS->GetInterface();
  }
  catch(...) {
    fprintf(stderr,"DSDummyTypeClient::Open: Allocation error\n");
    return(1);
  }

  // Create the database of filenames
  RamDB.CreateDatabase(ListFile);
  NoOfElements=RamDB.GetNoOfElements();

  // Read in each of the Descriptions
  try{
    // Make storage for all the pointers
    DBDescription = new GenericDS [NoOfElements];

    // Loop over the entries
    for(int ientry=0;ientry<NoOfElements;ientry++) {

      char path[MAXMEDIANAMELEN]; /* file name of DDL description*/

     // Initialiaze the Generic DS
      DBDescription[ientry] = GenericDS::CreateDescriptionRoot();

      // Get descriptor interface
      theDescriptionInterface=DBDescription[ientry].GetInterface();

      // Get DDL file name from data media filename and DDL file name
      strncpy(path,RamDB.GotoEntry(ientry)->GetFilename(),MAXMEDIANAMELEN-1);
      strncat(path,".dir/",MAXMEDIANAMELEN-1-strlen(path));
      strncat(path,DDLFile,MAXMEDIANAMELEN-1-strlen(path));
      path[MAXMEDIANAMELEN-1]=0;

      // Connect DDL file to coding scheme
      theCSInterface->SetDecoderFileName(path);

      // Connect descriptors to coding schemes
      theCSInterface->SetDescriptionInterface(theDescriptionInterface);

      // Now read this in
      theCS->StartDecode();
    }

  }
  catch(...) {
    fprintf(stderr,"DSDummyTypeClient::Open: Allocation error2\n");
    return(1);
  }

  // Perform the tidying up
  theCSInterface->destroy();

  return(0);
}

//----------------------------------------------------------------------------
unsigned long DSDummyTypeClient::Start(char *Query,int NoOfMatches)
{
  // Check data is sane
  assert(Query       != NULL);
  assert(NoOfMatches != 0);

  // --------------------- Initialise the query data -----------------------
  // Media
  MultiMedia *theMedia;
  MultiMediaInterfaceABC *theMediaInterface;

#ifdef __ImageProcessingTypeApplication
  MultiMedia *theDBMedia;
  MultiMediaInterfaceABC *theDBMediaInterface;
#endif

  // Create a list of all the best matches
  MatchList theMatchList;

  // Set up query and referece (DB) descriptions
  GenericDS theDescription = GenericDS::CreateDescriptionRoot();
  GenericDSInterfaceABC*             theDescriptionInterface = NULL;
  GenericDSInterfaceABC*             DBDescriptionInterface  = NULL;

  // Set up the extraction tool which will be needed for the query
  DSDummyTypeExtractionTool*         theExtractionTool      = NULL;
  DSDummyTypeExtractionInterfaceABC* theExtractionInterface = NULL;

  // Set up the search tool
  DSDummyTypeSearchTool*         theSearchTool              = NULL;
  DSDummyTypeSearchInterfaceABC* theSearchInterface         = NULL;

  try{
    // Create media object 
    theMedia = new MultiMedia;

    // obtain media interface
    theMediaInterface=theMedia->GetInterface();


#ifdef __ImageProcessingTypeApplication
    // Create media object for content set of database 
    theDBMedia = new MultiMedia;
    theDBMediaInterface=theDBMedia->GetInterface();
#endif /* __ImageProcessingTypeApplication*/

    // Obtain the descriptor interface
    theDescriptionInterface = theDescription.GetInterface();

    /* alternatively the query can also be a ddl description,
       then use GenericDSCS to gget query*/
    // Create extraction tool and connect description
    theExtractionTool = new DSDummyTypeExtractionTool(theDescriptionInterface);

    // Obtain the extraction interface from this extraction tool
    theExtractionInterface=theExtractionTool->GetInterface();

    // Create search tool
    theSearchTool = new DSDummyTypeSearchTool(theDescriptionInterface);

    // Obtain the search tool interface
    theSearchInterface=theSearchTool->GetInterface();

#ifdef __ImageProcessingTypeApplication
    // Set DBMedia to matching function
    theSearchInterface->SetMedia(theDBMediaInterface);
#endif /* __ImageProcessingTypeApplication*/
  }
  catch(...) {
    fprintf(stderr,"DSDummyTypeClient::Start: Allocation error1\n");
    return(1);
  }

  // Connect media to media data base
  RamDB.SetMedia(theMediaInterface);

  // If more matches are requested than the dbase has, limit it
  /* NoOfMatches can also be set to number of elements in RamDB*/
  if(NoOfMatches>RamDB.GetNoOfElements()) NoOfMatches=RamDB.GetNoOfElements();
  theMatchList.SetNumberOfMatches(NoOfMatches);

  // -------------------- Read in the query ---------------------------------

  // Names for elementary parts of the media sequence
  int i=0;              // part no of sequence
  char ppmimage[25];    // filename of single image (part of sequence)
  //char path[MAXMEDIANAMELEN]; // use if required for additional input files

  //cut sequence into parts
  if (!RamDB.DecodeNamedMPEGVideo(Query)) return (unsigned long) -1;

  // Get audio for video sequence
  // if (!RamDB.LoadAudioForVideo()) break;

  // Tell the extractor that this is the source medium
  theExtractionInterface->SetSourceMedia(theMediaInterface);

  // Reinitialise the D Type; maybe also audio D can be extracted here
  theExtractionInterface->InitExtracting();

  // Loop over the frames of the sequence
  while (1) {

    // Load frame of video sequnece
    sprintf(ppmimage,"m2v/%d.ppm",i);
    if (!RamDB.LoadNamedImage(ppmimage)) break; // break if end of sequence

    // check frame number of image
    {
      long FrameNo;

      FrameNo=theMediaInterface->GetImageTime();
      if (FrameNo == -1) FrameNo=i-1;
      theMediaInterface->SetImageTime(FrameNo);
    }

    // e.g., select background if region mask is available
    theMediaInterface->SelectVopAlphaLayer(BackgroundFromAlpha);

    // Extract D from media per frame of the input sequence
    theExtractionInterface->StartExtracting();

    // Increment frame counter
    i++;
  }

  // Collect partial results of the frame based extraction
  theExtractionInterface->PostExtracting();

  /// ------------------- Now do the matching -------------------------------

#ifdef __ImageProcessingTypeApplication
    // Set DBMedia to content database
    RamDB.SetMedia(theDBMediaInterface);
    // rewind counter of content set database
    RamDB.GotoEntry(0);
#endif /* __ImageProcessingTypeApplication*/

  for(int ientry=0;ientry<NoOfElements;ientry++) {

    int frame=0;                // frame counter of sequence

    // Get the interface of data base entry
    DBDescriptionInterface=DBDescription[ientry].GetInterface();

    // Tell the search interface which we're matching to
    theSearchInterface->SetRefDescriptionInterface(DBDescriptionInterface);

#ifdef __SearchAndRetrievalTypeApplication
    // Sort the result according to distance score
    theMatchList.InsertMatch(ientry,theSearchInterface->GetDistance());
#endif
#ifdef __ImageProcessingTypeApplication
    // Initialize processing
    theSearchInterface->InitSearch();

    // decode video sequence
    RamDB.SetMedia(theDBMediaInterface);
    if (!RamDB.DBDecodeMPEGVideo()) break;

    // Loop frames of video sequence
    while (1) {

      char ppmimage[25];    /* filename of single image*/

      // Get file name of frame
      sprintf(ppmimage,"m2v/%d.ppm",frame);

      // Load frame of video sequence
      if (!RamDB.LoadNamedImage(ppmimage)) break;

#ifdef XYUVDISP
      {
	MomVop *media=theDBMedia->GetImage();

	/* display input image*/
	xyuv_show_simple(0,(unsigned short *)media->y_chan->data->s,
			 (unsigned short *)media->u_chan->data->s,
			 (unsigned short *)media->v_chan->data->s,
			 datasize(media->y_chan)&1,
			 media->y_chan->x,media->y_chan->y,
			 media->u_chan->x,media->u_chan->y,
			 "Query");
      }
#endif

      theSearchInterface->StartSearch();
      /* or do something with respect to the return value*/

      frame++;
    }

    theMatchList.InsertMatch(ientry,theSearchInterface->PostSearch());
    /* or do something with respect to the return value*/
#endif /* __ImageProcessingTypeApplication*/
  }

  // -------------------- Display the best matches --------------------------
  for(int i=0;i<NoOfMatches;i++) {

    // Have we run out of entries?
    if(theMatchList.GetDBEntryNo(i)== (unsigned long) -1) break;

    // Display the file and score
    printf("%s\t%f",RamDB.GotoEntry(theMatchList.GetDBEntryNo(i))->
	   GetFilename(),
	   theMatchList.GetDistanceValue(i));

    // print file name to be displayed in GUI (optional)
    printf("\t%s.dir/0.jpg",RamDB.GotoEntry(theMatchList.GetDBEntryNo(i))->
	   GetFilename());

    // print other informtion

    // finish printed line
    printf("\n");
    
  }

  // Finally tidy up
  theExtractionInterface->destroy();
  theSearchInterface->destroy();

  return(0);
}

//----------------------------------------------------------------------------
unsigned long DSDummyTypeClient::Stop()
{
  // fprintf(stderr,"DSDummyTypeClient::Stop: Not Implemented");
  return 0;
}
//----------------------------------------------------------------------------
unsigned long DSDummyTypeClient::Close()
{
  // Free memories
  RamDB.FreeDatabase();

  if (DBDescription) delete [] DBDescription;

  NoOfElements =0;
  DBDescription =0;

  return 0;
}

	
//----------------------------------------------------------------------------
const UUID& DSDummyTypeClient::GetObjectID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DSDummyTypeClient::GetName(void)
{ return myName; }



