//////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Dummy author
// Dummy company
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DSDummyTypeSearch.cpp
//

#include <stdio.h>
#include <math.h>
#include <float.h>
#include "Descriptors.h"                     /* Naming Convention */
#include "SearchUtilities.h"

//=============================================================================
	

using namespace XM;


const UUID DSDummyTypeSearchInterface::myID = UUID("");
const char *DSDummyTypeSearchInterface::myName = "DSDummyTypeMatchingInterface";

const UUID DSDummyTypeSearchTool::myID = UUID("");
const char *DSDummyTypeSearchTool::myName = "DSDummyTypeMatchingTool";

const UUID DSDummyTypeSearchInterfaceABC::myID = UUID();

//=============================================================================
DSDummyTypeSearchInterfaceABC::DSDummyTypeSearchInterfaceABC()
{
  SetInterfaceID(myID);
}

//=============================================================================
DSDummyTypeSearchInterface::DSDummyTypeSearchInterface(DSDummyTypeSearchTool* aTool):
m_SearchTool(aTool)
{
  SetInterfaceID(myID);
}

//----------------------------------------------------------------------------
DSDummyTypeSearchInterface::~DSDummyTypeSearchInterface()
{
}

//----------------------------------------------------------------------------
const UUID& DSDummyTypeSearchInterface::GetInterfaceID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DSDummyTypeSearchInterface::GetName(void)
{ return myName; }


//----------------------------------------------------------------------------
void DSDummyTypeSearchInterface::destroy(void)
{ delete m_SearchTool; }

//----------------------------------------------------------------------------
int DSDummyTypeSearchInterface::
SetRefDescriptionInterface(GenericDSInterfaceABC
			   *RefDSDummyTypeDescriptionInterface)

{
  return m_SearchTool->
    SetRefDescriptionInterface(RefDSDummyTypeDescriptionInterface);
}

//----------------------------------------------------------------------------
GenericDSInterfaceABC* DSDummyTypeSearchInterface::
GetQueryDescriptionInterface(void)

{
  return m_SearchTool->GetQueryDescriptionInterface();
}

//----------------------------------------------------------------------------
int DSDummyTypeSearchInterface::
SetQueryDescriptionInterface(GenericDSInterfaceABC
			     *QueryDSDummyTypeDescriptionInterface)

{
  return m_SearchTool->
    SetQueryDescriptionInterface(QueryDSDummyTypeDescriptionInterface);
}

#ifdef __SearchAndRetrievalTypeApplication
//----------------------------------------------------------------------------
double DSDummyTypeSearchInterface::GetDistance(void)

{
  return m_SearchTool->GetDistance();
}
#endif

#ifdef __ImageProcessingTypeApplication
//----------------------------------------------------------------------------
int DSDummyTypeSearchInterface::SetMedia(MultiMediaInterfaceABC* media)

{
  return m_SearchTool->SetMedia(media);
}

//----------------------------------------------------------------------------
double DSDummyTypeSearchInterface::InitSearch(void)
{
  return m_SearchTool->InitSearch();
}

//----------------------------------------------------------------------------
double DSDummyTypeSearchInterface::StartSearch(void)
{
  return m_SearchTool->StartSearch();
}

//----------------------------------------------------------------------------
double DSDummyTypeSearchInterface::PostSearch(void)
{
  return m_SearchTool->PostSearch();
}

#endif /*__ImageProcessingTypeApplication*/

//=============================================================================
DSDummyTypeSearchTool::DSDummyTypeSearchTool():
m_Interface(this),
m_RefDescriptionInterface(0),
m_QueryDescriptionInterface(0)

{
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* create sub Searchs*/
  SubDSDummyTypeASearch *aSubDSDummyTypeASearch= new SubDSDummyTypeASearch();
  SubDSDummyTypeBSearch *aSubDSDummyTypeBSearch= new SubDSDummyTypeBSearch();

  if (aSubDSDummyTypeASearch)
    m_SubDSDummyTypeASearch=aSubDSDummyTypeASearch->GetInterface();
  if (aSubDSDummyTypeBSearch)
    m_SubDSDummyTypeBSearch=aSubDSDummyTypeBSearch->GetInterface();
#endif /* __HasSubTypes*/
  SetExposedInterface(&m_Interface);
}

//=============================================================================
DSDummyTypeSearchTool::
DSDummyTypeSearchTool(GenericDSInterfaceABC
		      *aQueryDescriptionInterface):
m_Interface(this),
m_RefDescriptionInterface(0),
m_QueryDescriptionInterface(0)

{
  /* connect Descritors with Searchs*/
  SetQueryDescriptionInterface(aQueryDescriptionInterface);

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* create sub Search */
  SubDSDummyTypeASearch *aSubDSDummyTypeASearch= new SubDSDummyTypeASearch();
  SubDSDummyTypeBSearch *aSubDSDummyTypeBSearch= new SubDSDummyTypeBSearch();

 if (aSubDSDummyTypeASearch)
   m_SubDSDummyTypeASearch=aSubDSDummyTypeASearch->GetInterface();
 if (aSubDSDummyTypeBSearch)
   m_SubDSDummyTypeBSearch=aSubDSDummyTypeBSearch->GetInterface();
 /* connection to description is done in GetDistance*/ 

#endif /* __HasSubTypes*/

  SetExposedInterface(&m_Interface);
}

//----------------------------------------------------------------------------
DSDummyTypeSearchTool::~DSDummyTypeSearchTool()
{
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDSDummyTypeASearch)
    m_SubDSDummyTypeASearch->destroy();
  if (m_SubDSDummyTypeBSearch)
    m_SubDSDummyTypeBSearc->destroy();
#endif /* __HasSubTypes*/
}

//----------------------------------------------------------------------------
const UUID& DSDummyTypeSearchTool::GetObjectID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DSDummyTypeSearchTool::GetName(void)
{ return myName; }

//----------------------------------------------------------------------------
bool DSDummyTypeSearchTool::SupportsPush(void)
{ return true; }

//----------------------------------------------------------------------------
bool DSDummyTypeSearchTool::SupportsPull(void)
{ return false; }

//----------------------------------------------------------------------------
int DSDummyTypeSearchTool::
SetRefDescriptionInterface(GenericDSInterfaceABC
			   *aRefDSDummyTypeDescriptionInterface)

{
  m_RefDescriptionInterface =  aRefDSDummyTypeDescriptionInterface;
  return 0;
}

//----------------------------------------------------------------------------
GenericDSInterfaceABC* DSDummyTypeSearchTool::
GetQueryDescriptionInterface(void)

{
  return m_QueryDescriptionInterface;
}

//----------------------------------------------------------------------------
int  DSDummyTypeSearchTool::
SetQueryDescriptionInterface(GenericDSInterfaceABC
		       *aQueryDSDummyTypeDescriptionInterface)

{

  m_QueryDescriptionInterface =  aQueryDSDummyTypeDescriptionInterface;
  
  /* could also look totally different if
     aQueryDSDummyTypeDescriptionInterface is not of DSDummyType node*/

  return 0;
}

#ifdef __SearchAndRetrievalTypeApplication
//----------------------------------------------------------------------------
double DSDummyTypeSearchTool::GetDistance(void)
{
  double dist;
  GenericDS DSDummyTypeRef,DSDummyTypeQuery;
  double FrameCntRef, FrameCntQuery;

  // Check it's all set up ok
  if(m_RefDescriptionInterface==NULL)   return(DBL_MAX);
  if(m_QueryDescriptionInterface==NULL) return(DBL_MAX);

  if(strcmp(m_RefDescriptionInterface->GetName(),
	    "GenericDSInterface") != 0) return(DBL_MAX);
  if(strcmp(m_QueryDescriptionInterface->GetName(),
	    "GenericDSInterface") != 0) return(DBL_MAX);

  // go through DS using:
  // - GetDescription("DSName",0)
  // - GetFirstChild()
  // - GetNextSibling()
  // - ...
  // check DS Type with:
  // - GetDSName()
  // - isNull()
  // and get values with:
  // - Get*Attribute
  // - Get*Value
  DSDummyTypeRef=m_RefDescriptionInterface->GetFirstChild();
  if (DSDummyTypeRef.GetDSName() != "DSDummyType") return(DBL_MAX);
  DSDummyTypeQuery=m_QueryDescriptionInterface->GetFirstChild();
  if (DSDummyTypeQuery.GetDSName() != "DSDummyType") return(DBL_MAX);
  
  DSDummyTypeRef.GetRealAttribute("FrameCounter",FrameCntRef);
  DSDummyTypeQuery.GetRealAttribute("FrameCounter",FrameCntQuery);
  dist=fabs(FrameCntRef - FrameCntQuery);

#ifdef __HasSubTypes /*get distances from Sub-DS*/
  m_SubDSDummyTypeASearch->
    SetQueryDescriptionInterface(GetMeFromPreviousBlock);
  m_SubDSDummyTypeASearch->
    SetRefDescriptionInterface(GetMeFromPreviousBlock);
  distance=m_SubDSDummyTypeASearch->GetDistance();
  m_SubDSDummyTypeBSearch->
    SetQueryDescriptionInterface(GetMeFromPreviousBlock);
  m_SubDSDummyTypeBSearch->
    SetRefDescriptionInterface(GetMeFromPreviousBlock);
  distance+=m_SubDSDummyTypeBSearch->GetDistance();
#endif /* __HasSubTypes*/

  // All done
  return(dist);
}
#endif /*__SearchAndRetrievalTypeApplication*/

#ifdef __ImageProcessingTypeApplication
//----------------------------------------------------------------------------
int DSDummyTypeSearchTool::SetMedia(MultiMediaInterfaceABC* media)

{
  if(media==NULL) return -1;
  m_Media=media;

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDSDummyTypeASearch)
    m_SubDSDummyTypeASearch->SetSourceMedia(media);
  if (m_SubDSDummyTypeASearch)
    m_SubDSDummyTypeASearch->SetSourceMedia(media);
#endif /* __HasSubTypes*/

  return 0;
}

//----------------------------------------------------------------------------
double DSDummyTypeSearchTool::InitSearch(void)
{
  double value=1.0;

  // Check it's all set up ok
  if(m_RefDescriptionInterface==NULL)   return(DBL_MAX);
  if(m_QueryDescriptionInterface==NULL) return(DBL_MAX);

  if(strcmp(m_RefDescriptionInterface->GetName(),
	    "GenericDSInterface") != 0) return(DBL_MAX);
  if(strcmp(m_QueryDescriptionInterface->GetName(),
	    "GenericDSInterface") != 0) return(DBL_MAX);

  // go through DS using:
  // - GetDescription("DSName",0)
  // - GetFirstChild()
  // - GetNextSibling()
  // - ...
  // check DS Type with:
  // - GetDSName()
  // - isNull()
  // and get values with:
  // - Get*Attribute
  // - Get*Value

#ifdef __HasSubTypes /*get distances from Sub-DS*/
  m_SubDSDummyTypeASearch->
    SetQueryDescriptionInterface(GetMeFromPreviousBlock);
  m_SubDSDummyTypeASearch->
    SetRefDescriptionInterface(GetMeFromPreviousBlock);
  value=m_SubDSDummyTypeASearch->InitSearch();
  m_SubDSDummyTypeBSearch->
    SetQueryDescriptionInterface(GetMeFromPreviousBlock);
  m_SubDSDummyTypeBSearch->
    SetRefDescriptionInterface(GetMeFromPreviousBlock);
  value+=m_SubDSDummyTypeBSearch->InitSearch();
#endif /* __HasSubTypes*/

  // All done
  return(value);
}

//----------------------------------------------------------------------------
double DSDummyTypeSearchTool::StartSearch(void)
{
  double value=1.0;

  //  Check it's all set up ok
  if(m_RefDescriptionInterface==NULL)   return(DBL_MAX);
  if(m_QueryDescriptionInterface==NULL) return(DBL_MAX);

  if(strcmp(m_RefDescriptionInterface->GetName(),
	    "GenericDSInterface") != 0) return(DBL_MAX);
  if(strcmp(m_QueryDescriptionInterface->GetName(),
	    "GenericDSInterface") != 0) return(DBL_MAX);

  // go through DS using:
  // - GetDescription("DSName",0)
  // - GetFirstChild()
  // - GetNextSibling()
  // - ...
  // check DS Type with:
  // - GetDSName()
  // - isNull()
  // and get values with:
  // - Get*Attribute
  // - Get*Value

#ifdef __HasSubTypes /*get distances from Sub-DS*/
  m_SubDSDummyTypeASearch->
    SetQueryDescriptionInterface(GetMeFromPreviousBlock);
  m_SubDSDummyTypeASearch->
    SetRefDescriptionInterface(GetMeFromPreviousBlock);
  value=m_SubDSDummyTypeASearch->StartSearch();
  m_SubDSDummyTypeBSearch->
    SetQueryDescriptionInterface(GetMeFromPreviousBlock);
  m_SubDSDummyTypeBSearch->
    SetRefDescriptionInterface(GetMeFromPreviousBlock);
  value+=m_SubDSDummyTypeBSearch->StartSearch();
#endif /* __HasSubTypes*/

  // All done
  return(value);
}

//----------------------------------------------------------------------------
double DSDummyTypeSearchTool::PostSearch(void)
{
  double value=1.0;

  // Check it's all set up ok
  if(m_RefDescriptionInterface==NULL)   return(DBL_MAX);
  if(m_QueryDescriptionInterface==NULL) return(DBL_MAX);

  if(strcmp(m_RefDescriptionInterface->GetName(),
	    "GenericDSInterface") != 0) return(DBL_MAX);
  if(strcmp(m_QueryDescriptionInterface->GetName(),
	    "GenericDSInterface") != 0) return(DBL_MAX);

  // go through DS using:
  // - GetDescription("DSName",0)
  // - GetFirstChild()
  // - GetNextSibling()
  // - ...
  // check DS Type with:
  // - GetDSName()
  // - isNull()
  // and get values with:
  // - Get*Attribute
  // - Get*Value

#ifdef __HasSubTypes /*get distances from Sub-DS*/
  m_SubDSDummyTypeASearch->
    SetQueryDescriptionInterface(GetMeFromPreviousBlock);
  m_SubDSDummyTypeASearch->
    SetRefDescriptionInterface(GetMeFromPreviousBlock);
  value=m_SubDSDummyTypeASearch->PostSearch();
  m_SubDSDummyTypeBSearch->
    SetQueryDescriptionInterface(GetMeFromPreviousBlock);
  m_SubDSDummyTypeBSearch->
    SetRefDescriptionInterface(GetMeFromPreviousBlock);
  value+=m_SubDSDummyTypeBSearch->PostSearch();
#endif /* __HasSubTypes*/

  // All done
  return(value);
}

#endif /*__ImageProcessingTypeApplication*/

//----------------------------------------------------------------------------
DSDummyTypeSearchInterfaceABC *DSDummyTypeSearchTool::GetInterface(void)
{ return &m_Interface; }

